% -------------------------------------------------------------------------
%
% analysis_sim
%
% Program to simulate idealised observations and background from given "truth"
% then try analysing them.
% Actual and assumed statistics are specified. 
% This version features: 
%   One type of observations, generated according to specified statistics
%   Background either a randomly perturbed or shifted version of truth.
%   Explicit calculation of matrix inverses
%
% P.J. Smith July 2014
% Apadpted from IDL/PWAVE procedure written by R. Swinbank 2002
% -------------------------------------------------------------------------
 
clear all

% options for call to random number generator
test   = 0; % 1 for printing extra info
norder = 2; % order for autoregressive correlation calculations (1 or 2)
  
% seeds for random no. generator 
% (so random errors are reproducible with same parameters)
seedo=1;
seedf=9;


% -----------------------------------------------------------------------------
% MAIN program
% -----------------------------------------------------------------------------

msgtext = {'**** ANALYSIS_SIM - Toy Analysis System ****',...
           '  ', ...
           'Constructs 1-D analysis from simulated truth, forecast and',...
           'observations with specified values and error characteristics',...
           '  ',...
           'You may change the analysis parameters at the prompt ...', ...
           '  ',...
           'To change ARRAYS you need to type in the correct number of new values',...
           '  '};
           
name = 'analysis_mobs';       

uiwait(msgbox(msgtext,name));


% ++++++++++
% BACKGROUND
% background (forecast) is based on "truth" shifted by "shift" units
shift = 0.0;
% OR perturbed according to these statistics
sigft = 2.0;
Lft   = 2.0;
biasf = 0.0;

% statistics used by test analysis -> assumed values = true values
sigf = sigft;
Lf   = Lft;

% ++++++++++
% OBSERVATIONS
% max no. of obs
nox = 11;
% initial observation locations
xo = [3.0 ,4.0 ,5.0 ,6.0 ,7.0]';
% actual no. of observations, all one type
no = 5;
% Observation error statistics
sigot = 1.0;
Lot   = 0.0;
biaso = 0.0;

% statistics used by test analysis -> assumed values = true values
sigo  = sigot;
Lo    = Lot;

% ++++++++

% set x values at which equations are solved
x0=0.;
xn=10.;
nx=100; % will have nx+1 points
dx=(xn-x0)/nx;
x=[x0:dx:xn]'; % x axis points

% initialise analysis vectors
xdim = length(x);

ft = zeros(size(x)); % truth
fb = zeros(size(x)); % background
a  = zeros(size(x)); % analysis

% DEFINE TRUTH
l_truth = menu_asl('Please select one of the following for the truth field:',...
    'zero everywhere', 'pseudo step function',...
    'sum of several sinusoidal curves');

ft = truth(x,l_truth,0);


% FORECAST STATISTICS
l_background = menu_asl('How do you want to generate the background field?',... 
     'background field is shifted relative to the "truth"',...
     'background equal to truth + random perturbation');

if (l_background ==1)
    
 name = 'BACKGROUND FIELD'; 
 Sprompt = {'Enter value of shift (+ve to left): '};
    
 Sanswer = inputdlg(Sprompt,name,1,{'0'}); 

if isempty(Sanswer)
  uiwait(msgbox({'*** user options cancelled ****','please try again ...'}))
  return
end
 
 shift = str2num(Sanswer{1});
 fb = truth(x,l_truth,shift);
 
end % l_background ==1


% REQUEST USER INPUT:
l_default = menu_asl('Use default parameter values?','Yes','No');

if (l_default==2)

% BACKGROUND ERROR STATISTICS   
 
 bgprompt={'Enter value of background error sigf:',...
           'Enter value of background error correlation length, Lf (put 0.0 for uncorrelated errors):'};
 
 name='BACKGROUND';
 defaultanswer={num2str(sigf),num2str(Lf)};
 bganswer=inputdlg(bgprompt,name,1,defaultanswer); 
 
 if isempty(bganswer)
  uiwait(msgbox({'*** user options cancelled ****','please try again ...'}))
  return
 else
  sigf = str2num(bganswer{1});
  Lf   = str2num(bganswer{2});     
 end
 
    
% OBSERVATION LOCATIONS & STATISTICS 
name = 'OBSERVATIONS'; 
Oprompt = {'Enter number of observations, (max 11, enter 0 if none): ',...
          'Enter locations of observations (values between 0 & 10)',...
          'Enter observation error, sigo: ',...
          'Enter observation error correlation length Lo (0.0 for uncorrelated errors):'};
df = {num2str(no),num2str(xo'),num2str(sigo),num2str(Lo)};

Obsinputok = 0;

while ~Obsinputok
    
Obsanswer = inputdlg(Oprompt,name,1,df); 

if isempty(Obsanswer)
  uiwait(msgbox({'*** user options cancelled ****','please try again ...'}))
  return
else
 no = str2num(Obsanswer{1});
 xo = str2num(Obsanswer{2})';

 if (~isempty(find(xo>10)) | ~isempty(find(xo<0)) )
  errorflag = 1;
 else
  errorflag = 0;
 end    
 
 sigo = str2num(Obsanswer{3});
 Lo   = str2num(Obsanswer{4});
end

if errorflag
 uiwait(msgbox('invalid observation locations/ values entered, please try again')); 
elseif (no~=0 & (no>nox | length(xo)~=no))
 uiwait(msgbox('no. of observations and no. of observation locations entered do not match, please try again')); 
else
 Obsinputok = 1;   
end    
 
end % while

end % if l_default==2

% generate background (forecast) field
if (l_background ==2)
 if Lf > 0.0 % correlated errors
  % call correlated random number generator 
  fb = ft + frand(x,sigf,Lf,norder,seedf,0,0);
 else   % uncorrelated errors
  randn('seed',seedf);   
  fb = ft + sigf*randn(size(x));  
 end
end


% Single set of obs with specified sigma and error correlations
ob = zeros(no,1); % y
fo = zeros(no,1); % Hfb
od = zeros(no,1); % y-Hfb 

% Generate observation values from truth
if no > 0
 if (Lo > 0.0)
  ox = ft + frand(x,sigo,Lo,norder,seedo,0,0);
  ob = interp1(x,ox,xo,'linear');   
 else % uncorrelated errors
  ob = interp1(x,ft,xo,'linear');   
  randn('seed',seedo);
  ob = ob + sigo*randn(no,1);
 end


% forecast fields at observation locations & y-Hfb differences
fo = interp1(x,fb,xo,'linear'); % Hfb
od = ob - fo; % y - Hfb

% Set up M = HBH^T + R
M = zeros(no);

% background correlations in observation space (HBH^T)
rho = soar2(xo,xo,Lf);      
M   = sigf*sigf*rho;
 
% add observation correlations R (no x no)
rho = soar2(xo,xo,Lo);
M   = M + sigo*sigo*rho; % (HBH^T + R)

% get inverse of M, (HBH^T + R)^-1
Mi=inv(M);

% evaluate analysis
      
rho = soar2(x,xo,Lf);   
cf  = sigf*sigf*rho;  % BH^{T}, (nx x no)

z   = Mi*od;     % (HBH^T + R)^-1*(y-Hfb), (no x 1)
    
a   = fb + cf*z;  % xa = fb + BH^T(HBH^T + R)^-1*(y-Hfb)

else
    
 a = fb;  
 
end
% plot results
h = figure; hold on
title('analysis\_sim')

xtext = ['sigo = ' num2str(sigo) ', Lo = ' num2str(Lo) ', sigf = ' num2str(sigf) ', Lf = ' num2str(Lf)];

plot(x,ft,'k','linewidth',2)   % truth
plot(x,fb,'b:','linewidth',2)   % background
if (no > 0)
 plot(xo,ob,'kx','linewidth',1,'MarkerSize',8)  % observations
end
plot(x,a,'g--','linewidth',2)   % analysis
set(gca,'xtick',[0:2:10],'xlim',[0 10],'ylim',[-10 10], 'ytick',[-10:5:10],'XMinorTick','on','YMinorTick','on');
box on 

xlabel(xtext)
legend('truth','background','observations','analysis')

% save plot upon request

l_save = menu_asl('Do you wish to save this plot?','Yes','No');

if (l_save==1)
 prompt = {'please enter a filename:'};
 name   = 'save figure';
 df     = {'analysis_sim_plot1'} ;
 pname  = inputdlg(prompt,name,1,df);
 
 if isempty(pname)
  disp('*** user options cancelled ****')
  disp('please try again ...') 
  break
 end    
 
 format = 'fig';
 saveas(h,char(pname),format);   
    
end

% Write out options chosen
%
disp('  ')
disp('********************************************************************')
disp('  ')
disp('SUMMARY OF CHOSEN PARAMETERS:')
disp('  ')

switch l_truth
 case 1 
   disp('truth is zero everywhere')
 case 2 
   disp('truth is pseudo step function')
 case 3 
   disp('truth is sum of several sinusoidal curves')
end
disp('  ')
switch l_background
 case 1 
   disp('background field is shifted "truth"')  
 case 2
   disp('background equal to truth + random perturbation')
end
disp('  ')
disp(['background error standard deviation, sigf = ' num2str(sigf)])
disp(['background error correlation lengthscale, Lf   = ' num2str(Lf)])
disp('  ')
disp(['number of observations: ' num2str(no)])
disp(['observation locations : ' num2str(xo')])
disp(['observation values    : ' num2str(ob')])
disp(['observation error standard deviation, sigo = ' num2str(sigo)])
disp(['observation error correlation lengthscale, Lo   = ' num2str(Lo)])
disp('  ')
disp('********************************************************************')
%%%%%%%%%%  END OF PROGRAM  %%%%%%%%%%

